#if 0
/*
General Description:
This file contains assembler code which is shared among the USB driver
implementations for different CPU cocks. Since the code must be inserted
in the middle of the module, it's split out into this file and #included.

Jump destinations called from outside:
	sofError: Called when no start sequence was found.
	se0: Called when a package has been successfully received.
	overflow: Called when receive buffer overflows.
	doReturn: Called after sending data.

Outside jump destinations used by this module:
	waitForJ: Called to receive an already arriving packet.
	sendAckAndReti:
	sendNakAndReti:
	sendCntAndReti:
	usbSendAndReti:

*/

#define token	x1

ignorePacket:
	clr	token
	rjmp	storeTokenAndReturn

;----------------------------------------------------------------------------
; Processing of received packet (numbers in brackets are cycles after center of SE0)
;----------------------------------------------------------------------------
;This is the only non-error exit point for the software receiver loop
;we don't check any CRCs here because there is no time left.
se0:
	subi	cnt, USB_BUFSIZE	;[5]
	neg	cnt			;[6]
	sub	YL, cnt			;[7]
	sbci	YH, 0			;[8]
	nop2				;[10]
	ld	token, y		;[11]
	cpi	token, USBPID_DATA0	;[13]
	breq	handleData		;[14]
	cpi	token, USBPID_DATA1	;[15]
	breq	handleData		;[16]
	lds	shift, usbDeviceAddr	;[17]
	ldd	x2, y+1			;[19] ADDR and 1 bit endpoint number
	lsl	x2			;[21] shift out 1 bit endpoint number
	cpse	x2, shift		;[22]
	rjmp	ignorePacket		;[23]
	cpi	token, USBPID_IN	;[27]
	breq	handleIn		;[28]
	cpi	token, USBPID_SETUP	;[29]
	breq	handleSetupOrOut	;[30]
	cpi	token, USBPID_OUT	;[31]
	brne	ignorePacket		;[32] must be ack, nak or whatever

doReturn:
	reti

handleData:
	lds	shift, usbCurrentTok	;[18]
	tst	shift			;[20]
	breq	doReturn		;[21]
	lds	x2, usbRxLen		;[22]
	tst	x2			;[24]
	brne	sendNakAndReti		;[25]
; 2006-03-11: The following two lines fix a problem where the device was not
; recognized if usbPoll() was called less frequently than once every 4 ms.
	cpi	cnt, 4			;[26] zero sized data packets are status phase only -- ignore and ack
	brmi	sendAckAndReti		;[27] keep rx buffer clean -- we must not NAK next SETUP
#if USB_CFG_CHECK_DATA_TOGGLING
	sts	usbCurrentDataToken, token	; store for checking by C code
#endif
	sts	usbRxLen, cnt		;[28] store received data, swap buffers
	sts	usbRxToken, shift	;[30]
	lds	x2, usbInputBufOffset;[32] swap buffers
	ldi	cnt, USB_BUFSIZE	;[34]
	sub	cnt, x2			;[35]
	sts	usbInputBufOffset, cnt;[36] buffers now swapped
	rjmp	sendAckAndReti		;[38] 40 + 17 = 57 until SOP

handleIn:
;We don't send any data as long as the C code has not processed the current
;input data and potentially updated the output data. That's more efficient
;in terms of code size than clearing the tx buffers when a packet is received.
	lds	x1, usbRxLen		;[30]
	cpi	x1, 1			;[32] negative values are flow control, 0 means "buffer free"
	brge	sendNakAndReti		;[33] unprocessed input packet?
	ldi	x1, USBPID_NAK		;[34] prepare value for usbTxLen
#if USB_CFG_HAVE_INTRIN_ENDPOINT
	andi	x3, 0xf			;[35] x3 contains endpoint
#if USB_CFG_SUPPRESS_INTR_CODE
	brne	sendNakAndReti		;[36]
#else
	brne	handleIn1		;[36]
#endif
#endif
	lds	cnt, usbTxLen		;[37]
	sbrc	cnt, 4			;[39] all handshake tokens have bit 4 set
	rjmp	sendCntAndReti		;[40] 42 + 16 = 58 until SOP
	sts	usbTxLen, x1		;[41] x1 == USBPID_NAK from above
	ldi	YL, lo8(usbTxBuf)	;[43]
	ldi	YH, hi8(usbTxBuf)	;[44]
	rjmp	usbSendAndReti		;[45] 57 + 12 = 59 until SOP
#endif
