#pragma once

#define CONCAT_(A,B) A ## B
#define CONCAT(A,B) CONCAT_(A,B)
#define PORT_(suffix) CONCAT(PORT,suffix)
#define PIN_(suffix) CONCAT(PIN,suffix)
#define DDR_(suffix) CONCAT(DDR,suffix)

#include "usbhostconfig.h"

// HOST ist 0 oder 1 vom Makefile
// USBHOSTnD,USBHOSTnP,USBHOSTnN von der Konfigurationsdatei
#define USBOUT PORT_(CONCAT(CONCAT(USBHOST,HOST),D))
#define USBDDR  DDR_(CONCAT(CONCAT(USBHOST,HOST),D))
#define USBIN   PIN_(CONCAT(CONCAT(USBHOST,HOST),D))
#define USBPLUS  CONCAT(CONCAT(USBHOST,HOST),P)
#define USBMINUS CONCAT(CONCAT(USBHOST,HOST),M)
#define USBIDLE  (1<<(USBMINUS))
#define USBMASK  (1<<(USBPLUS)|1<<(USBMINUS))
#define USBQUERY CONCAT(usbQuery,HOST)

// register names
#define x1	r16
#define x2	r17
#define shift	r18
#define cnt	r19
#define x3	r20
#define x4	r21
#define x5	r22
#define bitcnt	x5
#define phase	x4
#define leap	x4

#ifdef __ASSEMBLER__

#define __SFR_OFFSET 0
#include <avr/io.h>
#define nop2 rjmp .
#define USB_BUFSIZE 11

#else

#include <avr/io.h>
typedef unsigned char byte;
#define elemof(x) (sizeof(x)/sizeof(*(x)))

register unsigned isrSave asm("r2");
register unsigned isrTick asm("r4");
#define isrFrac GPIOR1	// Gebrochener Teil des 1-kHz-Generators

namespace usb{
enum{
 PID_SETUP	= 0x2d,	// host=>device, data[8]
 PID_OUT	= 0xe1,	// host=>device, data[0]
 PID_IN		= 0x69,	// host=>device, data[0]
 PID_DATA0	= 0xc3,	// both, data[0..8]
 PID_DATA1	= 0x4b,	// both, data[0..8]
 PID_ACK	= 0xd2,	// ?
 PID_NAK	= 0x5a,	// ?
 PID_STALL	= 0x1e,	// ?
};
}

// EP0Size must be 8
// The one-and-only Interrupt In endpoint can have any number between 1 and 15
// Its endpoint size must be ≤ 8 (bytes) but actual data can be longer
// An Interrupt Out endpoint is not supported
struct UsbHost{
 enum{
  stDisconn,	// abgesteckt (Bus permanent SE0)
  stConn,	// angesteckt, Pause vor USB-Reset
  stReset,	// USB-Reset, weiter mit stRest
  stFromLowPower,// Setup-Transfer zum Deaktivieren des Remote-Wakeup-Features
		// (signed char)state >=3 allesamt mit SOF
		// Weiter zu stConfigured, bei Fehler zu stConn
  stRest,	// Pause nach USB-Reset (Device hochlaufen lassen)
  stDeviceDesc,	// Gerätebeschreiber anfragen (Wenn nichts angegeben gehen alle Zustände zum nächsten über)
  stAddress,	// Adresse setzen
  stConfigDesc,
  stHidRepDesc,
  stConfigured,	// Normalzustand, mit SOF
  stToLowPower,	// gehe zu Low Power und aktiviere ggf. Remote Wakeup per SETUP-Transfer, (noch) mit SOF
  stStringDesc0,// weiter zu stConfigured
  stStringDesc1,
  stStringDesc2,
  stStringDesc3,

  stLowPower0=128,	// ohne Remote Wakeup
  stLowPower1,	// mit Remote Wakeup: Zustand K (1..15 ms) geht upstream
  stKState,	// 20 ms K-Zustand (auf allen drei(!) Ports)
		// Am Ende EOP (2 Bitzeiten SE0, dann J).
		// Weiter zu stFromLowPower
// Die folgenden Zustände können NUR durch Busaktivität verlassen werden
  stFullSpeed,	// Full-Speed-Gerät detektiert (permanent K)
  stDefekt,	// Busleitungen kleben an H (permanent SE1)
  stNoDeviceDesc,	// Defektzustände = Zustand+128 (Bus permanent J)
  stNoAddress,
  stNoConfigDesc,
  stNoHidRepDesc,
  stNoConfigured,
  stNoToLowPower,
  stNoStringDesc0,
  stNoStringDesc1,
  stNoStringDesc2,
  stNoStringDesc3,
  stNoHidDevice,// kein HID-Gerät
  stNoMemory,	// Deskriptor passt nicht in den RAM
  stWeirdDesc,	// Sonstiges Problem beim Parsen der Deskriptoren
 };
 byte state;		// 0..31
 byte state_ms;		// Verweildauer in einem Zustand, bspw. USB-Reset-Zähler (bis 50 ms ist das USB-Reset lang)
 byte EPI_n;		// Nummer des Interrupt-In-Endpoints
 byte EPI_t;		// Abfrageintervall des Interrupt-In-Endpoints
 char TxLen;
 char RxLen;
 byte TxBuf[11];	// PID + data[0..8] + crc16
 byte RxBuf[11];	// PID + data[0..8] + crc16
 byte*DeviceDesc;
 byte*ConfigDesc;	// RAM-Fresser, dynamisch?
 byte*HidReportDesc;	// RAM-Fresser, dynamisch?
 static void init();
 static void sof();
 void poll();
};

extern UsbHost usbhost0,usbhost1;

unsigned usbCrc16(const byte*data, byte len);
void usbCrc16Append(byte*data, byte len);

inline unsigned GetTickCount() {
 unsigned ret;
 asm(" movw %0,r4":"=r"(ret));	// wichtig: atomare Operation ohne Interruptunterbrechung
 return ret;
}

#define USBDESCR_HID            0x21
#define USBDESCR_HID_REPORT     0x22
#define USBDESCR_HID_PHYS       0x23
#define USBATTR_SELFPOWER       0x40
#define USBATTR_REMOTEWAKE      0x20
#define USBRQ_HID_GET_REPORT    0x01
#define USBRQ_HID_GET_IDLE      0x02
#define USBRQ_HID_GET_PROTOCOL  0x03
#define USBRQ_HID_SET_REPORT    0x09
#define USBRQ_HID_SET_IDLE      0x0a
#define USBRQ_HID_SET_PROTOCOL  0x0b

#endif
